import java.awt.Color;
import java.text.DateFormat;

import javax.print.PrintService;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.PageRanges;
import javax.swing.UIManager;

import jpdf.attribute.*;
import jpdf.gui.dialog.PrintDialog;
import jpdf.gui.dialog.PrintDialogPreferences;
import jpdf.gui.dialog.factory.SetupContentPrintDialogFactory;
import jpdf.gui.preview2.PageLayout;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.manage.PrintParameterHolder;
import jpdf.manage.ResourceAdministrator;
import jpdf.printer.TableComponentPrinter;
import jpdf.printer.swing.PrintTableUtility;
import jpdf.utility.PrintUtility;


/**
 * Class PreviewTableApplication implements an application which previews and prints a 
 * Swing Table.  The table is displayed in a JPDF Preview Dialog.  Page Setup and Print 
 * dialogs can be displayed from the Preview Dialog.
 *
 * <P>This class provides an example of how to use the JPDF to implement printing capabilities.
 * The table is not displayed on the screen to demonstrate the ability to preview and print
 * off-screen Swing Components.
 */
public class PreviewTableApplication
{
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final TableComponentPrinter TABLE_PRINTER;
  private static final String                TITLE = "Massachusetts Libraries";
  
  
  //-----------------------------------------------------------------------
  // Class Initialization.
  //-----------------------------------------------------------------------
  static
  {
    // Set the Look-And-Feel.
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception theEx) {}  
    
    // Create the table after the Look and Feel has been set.
    MassLibTable theTable = new MassLibTable();
    
    // Make sure that the table uses its preferred column widths.
    // NOTE: This needs to be done because the table is not being displayed on the 
    //       screen.  If it was displayed on the screen, then Swing would do it.
    PrintTableUtility.setToPreferredColumnWidths(theTable); 
    
    // Create the Printer.  No need to create a Table clone since the table is not 
    // displayed on the screen.  The Table is configured for printing.
    TABLE_PRINTER = new TableComponentPrinter(theTable, TITLE, false, true);  
  }
  
  
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Application entry point.
   */
  public static void main(String theParams[]) 
  {
    // Initialize the JPDF Environment.
    initializeJpdfEnvironment();
    
    // Display the Application Window.  
    (new DisplayFrame()).setVisible(true);
  }      

   
  //-----------------------------------------------------------------------
  // Protected methods.
  //-----------------------------------------------------------------------

  /**
   * Displays a Preview Window which contains a preview display of the table.
   */
  protected static void displayPreviewWindow(DisplayFrame theParentFrame) 
  {
      PreviewPrintableDialog thePreviewDialog = null;
      
      // Create and display the Preview Dialog.
      // NOTE: The Dialog Factory creates the Page Setup and Print dialogs that can 
      //       be displayed from within the Preview Dialog.
      thePreviewDialog = new PreviewPrintableDialog(theParentFrame, TABLE_PRINTER);
      thePreviewDialog.setDialogFactory(new SetupContentPrintDialogFactory()); 
      thePreviewDialog.show();  
  }
  
 /**
   * Displays a Print Dialog which allows the table to be printed.
   */
  protected static void displayPrintDialog(DisplayFrame theParentFrame) 
  {
    PrintRequestAttributeSet theAttributeSet = PrintManager.getDefaultPrintAttributes();
    PrintService             thePrintService = null;
    PrintDialog              theDialog = null;
    
    // Create and display the Print Dialog.
    theDialog = new PrintDialog(theParentFrame, theAttributeSet, true); 
    theDialog.show(); 

    // Get the Selected Print Service.
    // NOTE: Null is returned if the user did not press the Print Button.
    thePrintService = theDialog.getSelectedPrintService();
    if (thePrintService != null) 
    {
      // Submit a Print Job.
      PrintManager.printPrintable(TABLE_PRINTER, theDialog.getPrintInfoHolder());
    }
  }
  
  /**
   * Exits the application.
   */
  protected static void exitApplication() 
  {
    // Shutdown the JPDF Environment.
    PrintManager.shutdownEnvironment();
    
    // Exit.
    System.exit(0); 
  }
  
  
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Initializes the JPDF environment.
   */
  private static void initializeJpdfEnvironment() 
  {
    final boolean IS_LANDSCAPE = false;
    final float   MARGIN_INCHES = 0.4f;
    final float   HEAD_NOTE_FONT_SIZE = 24f;
    final float   FOOT_NOTE_FONT_SIZE = 12f;
    
    PrintRequestAttributeSet  theAttributeSet = null; 
    String                    theDateAndTimeMarker = null;
    String                    thePageAndCountMarker = null;
    
    // Initialize the JPDF Environment.
    try
    {
      PrintManager.initializeEnvironment();
    }
    catch (Throwable theEx)
    {
      theEx.printStackTrace();
      System.exit(99); 
    }  
    
    // Preload the list of media supported by the Default Print Service on a background thread.  
    // This speeds up the initial display of the Page Setup dialog.
    PrintManager.preloadPrintServiceMediaOnThread();
    
    // Specify the Date Format used in Page Headers/Footers.
    ResourceAdministrator.setTimeStyle(DateFormat.SHORT);
    
    // Print Service selection will be done using icons.
    PrintDialogPreferences.setToUsePrintServiceSelectionIcons(true);

    // Do not include the Banner Page Controller in the Print Dialogs.
    PrintDialogPreferences.setToIncludeBannerPageController(false);
    
    // Set the Preview Dialog preferences.
    PreviewPrintableDialog.setToUsePreviousSizeLocation(true);
    PreviewPrintableDialog.setInitialPageLayout(PageLayout.FULL_GRID);
    PreviewPrintableDialog.setPageNavigationAudible(true);
    PreviewPrintableDialog.setInitialZoomFitWidth();
    
    // Get the Default Attribute Set.
    theAttributeSet = PrintManager.getDefaultPrintAttributes(); 
    
    // Specify the Page Orientation and Margins.
    PrintUtility.updateAttributeSet(IS_LANDSCAPE, MARGIN_INCHES, theAttributeSet, false);
    
    // Add the initial attributes we want to use for printing.
    // NOTE: Special markers are used in the Page Header/Footer.  These are replaced by actual values
    //       when the material is printed.
    theDateAndTimeMarker = PrintParameterHolder.DATE_MARKER + " " + PrintParameterHolder.TIME_MARKER;
    thePageAndCountMarker = "Page " + PrintParameterHolder.PAGE_NUMBER_MARKER + " of " + 
                            PrintParameterHolder.PAGE_COUNT_MARKER;
    theAttributeSet.add(new JobName("Preview Table Example", null));
    theAttributeSet.add(new PageHeadNotes("", PrintParameterHolder.TITLE_MARKER, "", Color.BLUE, 
                                          HEAD_NOTE_FONT_SIZE));
    theAttributeSet.add(new PageFootNotes(theDateAndTimeMarker, "", thePageAndCountMarker, Color.BLUE, 
                                          FOOT_NOTE_FONT_SIZE));
    theAttributeSet.add(new OffsetTop(0.1f, true));
    theAttributeSet.add(new AlignmentFactorHorizontal(0.5f));
    theAttributeSet.add(PageFrame.TRUE);
    theAttributeSet.add(PageContentFrame.TRUE);
    theAttributeSet.add(UseBackgroundColor.FALSE);
    theAttributeSet.add(BackgroundColorOnPage.FALSE);
    theAttributeSet.add(ContentHeadersOnEveryPage.TRUE);
    theAttributeSet.add(SplitColumns.FALSE);
    theAttributeSet.add(new PageRanges(1,2));  
  }      
}